import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.awt.image.*;

public class ImageCanvas extends Canvas
                         implements ClipboardOwner, KeyListener
{
    ExtendedClipboard   clipboard;
    Image               img;
    MediaTracker        tracker = new MediaTracker(this);

    int                 CF_DIB = 8;

    public ImageCanvas ()
    {
        addKeyListener(this);
        clipboard = new NativeExtendedClipboard();
    }

    public void setImage (Image img)
    {
        tracker.addImage(img, 0);
        try
        {
            tracker.waitForAll();
        }
        catch (Exception e)
        {
            System.out.println("Error loading image: " + e.getMessage());
            return;
        }

        if (tracker.isErrorAny())
        {
            System.out.println("Error loading image.");
            return;
        }

        this.img = img;
        repaint();
    }

    public void paint (Graphics g)
    {
        if (img != null)
        {
            Dimension d = this.getSize();
            g.setColor(Color.white);
            g.fillRect(0,0, d.width, d.height);     // blank background
            g.drawImage(img, 0, 0, img.getWidth(this), img.getHeight(this), this);
        }
    }

    public void keyPressed (KeyEvent evt)
    {
        if (evt.isControlDown())
        {
            if (evt.getKeyCode() == KeyEvent.VK_A)  // copy command
            {
                try
                {
                    int width = img.getWidth(this);
                    int height = img.getHeight(this);
                    int[] pixels = new int[width * height];
                    PixelGrabber pg = new PixelGrabber(img, 0, 0, width, height, pixels, 0, width);
                    pg.grabPixels(0);
                    System.out.print(width);
                    System.out.print(height);
                    System.out.print(width * height);
                    for (int i = 0; i < pixels.length; i++)
                        System.out.print(pixels[i]);

                    clipboard.setContents(new ExtendedTransferableImage(width, height, pixels), this);
                }
                catch (Exception e)
                {
                    System.out.println("Error copying data. Error: " + e.getMessage());
                }
                evt.consume();
            }
            else if (evt.getKeyCode() == KeyEvent.VK_B) // paste command
            {
                ExtendedTransferable clipboardData = clipboard.getContents(this);
                if (clipboardData != null)
                {
                    int[] formats = clipboardData.getFormats();
                    for (int i = 0; i < formats.length; i++)
                        System.out.println("format " + i + ": " + formats[i]);


                    byte[] imageData = clipboardData.getBytes(CF_DIB);

                    for (int i = 0; i < imageData.length; i+=4)
                        //System.out.print(imageData[i]);
                        System.out.print(getIntFromBytes(imageData, i));
                    System.out.println();
                    int width = getIntFromBytes(imageData, 0);
                    int height = getIntFromBytes(imageData, 4);
                    int numPixels = getIntFromBytes(imageData, 8);
                    System.out.println("width: " + width);
                    System.out.println("height: " + height);
                    System.out.println("numPixels: " + numPixels);

                    int[] pixels = new int[(imageData.length - 12) / 4];
                    for (int i = 0; i < pixels.length; i++)
                        pixels[i] = getIntFromBytes(imageData, 12 + (i * 4));

                    MemoryImageSource mis = new MemoryImageSource(width, height, pixels, 0, width);
                    this.setImage(Toolkit.getDefaultToolkit().createImage(mis));
                }
                evt.consume();
            }
        }
    }

    public void keyReleased (KeyEvent evt)  {   }
    public void keyTyped (KeyEvent evt)     {   }

    public void lostOwnership(Clipboard clipboard, Transferable contents)
    {   }

    public static void main (String args[])
    {
        Frame f = new Frame("Image Copy");
        f.setLayout(new GridLayout(0,1));

        ImageCanvas canvas1 = new ImageCanvas();
        ImageCanvas canvas2 = new ImageCanvas();

        canvas1.setImage(Toolkit.getDefaultToolkit().getImage("images/cloud.gif"));
        canvas2.setImage(Toolkit.getDefaultToolkit().getImage("images/sun_point.gif"));

        f.add(canvas1);
        f.add(canvas2);

        f.setSize(300,300);
        f.show();
    }

    private int getIntFromBytes (byte[] b, int offset)
    {
        return (((int)b[offset + 3]) << 24 | ((int)b[offset + 2]) << 16 | ((int)b[offset + 1]) << 8 | ((int)b[offset]));
    }

}

class ExtendedTransferableImage implements ExtendedTransferable
{
    static int  CF_DIB = 8;
    byte[]      imageData;
    byte[]      bytesForInt;

    public ExtendedTransferableImage (int width, int height, int[] pixels) throws Exception
    {
        // format we want to transfer is width, height, numPixels, pixels
        imageData = new byte[(pixels.length + 3) * 4];
        insertIntIntoByteArray(imageData, width, 0);
        insertIntIntoByteArray(imageData, height, 4);
        insertIntIntoByteArray(imageData, pixels.length, 8);

        int offset = 12;
        for (int i = 0; i < pixels.length; i++)
        {
            insertIntIntoByteArray2(imageData, pixels[i], offset);
            offset += 4;
        }
    }

    public int[] getFormats ()
    {
        int formats[] = new int[1];
        formats[0] = CF_DIB;
        return (formats);
    }

    public byte[] getBytes(int format)
    {
        if (format == CF_DIB)
            return (imageData);
        else
            return (null);
    }

    public Object getTransferData(DataFlavor flavor)
    {
        if (ExtendedClipboard.extendedFlavor.equals(flavor))
            return (imageData);
        else
            return (null);
    }

    public DataFlavor[] getTransferDataFlavors()
    {
        DataFlavor[] flavors = new DataFlavor[1];
        flavors[0] = ExtendedClipboard.extendedFlavor;
        return (flavors);
    }

    public boolean isDataFlavorSupported(DataFlavor flavor)
    {
        return (ExtendedClipboard.extendedFlavor.equals(flavor));
    }

    private void insertIntIntoByteArray(byte[] data, int i, int offset)
    {
        data[offset] = (byte) (i >> 0);
        data[offset + 1] = (byte) (i >> 8);
        data[offset + 2] = (byte) (i >> 16);
        data[offset + 3] = (byte) (i >> 24);
    }

    private void insertIntIntoByteArray2(byte[] data, int i, int offset)
    {
        data[offset] = (byte) (i >> 24);
        data[offset + 1] = (byte) (i >> 16);
        data[offset + 2] = (byte) (i >> 8);
        data[offset + 3] = (byte) (i >> 0);
    }

}
